package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.alibaba.excel.event.AnalysisEventListener;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.ctop.entity.BytedanceCustomerDdTbtjData;
import org.jeecg.modules.ctop.service.IBytedanceCustomerDdTbtjDataService;
import org.jeecg.modules.excelutil.BytedanceCustomTbtjDataExcelListener;
import org.jeecg.modules.excelutil.ExcelUtils;
import org.jeecg.modules.excelutil.entity.BytedanceCustomTbtjDataEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Map;

/**
 * 淘宝特价版客户反馈数据
 * @author jeecg-boot
 * @date   2021-04-22
 * @version V1.0
 */
@Slf4j
@Api(tags="淘宝特价版客户反馈数据")
@RestController
@RequestMapping("/ctop/bytedanceCustomerDdTbtjData")
public class BytedanceCustomerDdTbtjDataController {
	@Autowired
	private IBytedanceCustomerDdTbtjDataService bytedanceCustomerDdTbtjDataService;

	/**
	  * 分页列表查询
	 * @param bytedanceCustomerDdTbtjData
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@ApiOperation(value="淘宝特价版客户反馈数据-分页列表查询", notes="淘宝特价版客户反馈数据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<BytedanceCustomerDdTbtjData>> queryPageList(BytedanceCustomerDdTbtjData bytedanceCustomerDdTbtjData,
																	@RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
																	@RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
																	HttpServletRequest req) {
		Result<IPage<BytedanceCustomerDdTbtjData>> result = new Result<>();
		QueryWrapper<BytedanceCustomerDdTbtjData> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceCustomerDdTbtjData, req.getParameterMap());
		queryWrapper.orderByDesc("state_date");
		Page<BytedanceCustomerDdTbtjData> page = new Page<BytedanceCustomerDdTbtjData>(pageNo, pageSize);
		IPage<BytedanceCustomerDdTbtjData> pageList = bytedanceCustomerDdTbtjDataService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param bytedanceCustomerDdTbtjData
	 * @return
	 */
	@AutoLog(value = "淘宝特价版客户反馈数据-添加")
	@ApiOperation(value="淘宝特价版客户反馈数据-添加", notes="淘宝特价版客户反馈数据-添加")
	@PostMapping(value = "/add")
	public Result<BytedanceCustomerDdTbtjData> add(@RequestBody BytedanceCustomerDdTbtjData bytedanceCustomerDdTbtjData) {
		Result<BytedanceCustomerDdTbtjData> result = new Result<>();
		try {
			bytedanceCustomerDdTbtjDataService.save(bytedanceCustomerDdTbtjData);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param bytedanceCustomerDdTbtjData
	 * @return
	 */
	@AutoLog(value = "淘宝特价版客户反馈数据-编辑")
	@ApiOperation(value="淘宝特价版客户反馈数据-编辑", notes="淘宝特价版客户反馈数据-编辑")
	@PutMapping(value = "/edit")
	public Result<BytedanceCustomerDdTbtjData> edit(@RequestBody BytedanceCustomerDdTbtjData bytedanceCustomerDdTbtjData) {
		Result<BytedanceCustomerDdTbtjData> result = new Result<BytedanceCustomerDdTbtjData>();
		BytedanceCustomerDdTbtjData bytedanceCustomerDdTbtjDataEntity = bytedanceCustomerDdTbtjDataService.getById(bytedanceCustomerDdTbtjData.getId());
		if(bytedanceCustomerDdTbtjDataEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = bytedanceCustomerDdTbtjDataService.updateById(bytedanceCustomerDdTbtjData);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "淘宝特价版客户反馈数据-通过id删除")
	@ApiOperation(value="淘宝特价版客户反馈数据-通过id删除", notes="淘宝特价版客户反馈数据-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			bytedanceCustomerDdTbtjDataService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "淘宝特价版客户反馈数据-批量删除")
	@ApiOperation(value="淘宝特价版客户反馈数据-批量删除", notes="淘宝特价版客户反馈数据-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<BytedanceCustomerDdTbtjData> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<BytedanceCustomerDdTbtjData> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.bytedanceCustomerDdTbtjDataService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "淘宝特价版客户反馈数据-通过id查询")
	@ApiOperation(value="淘宝特价版客户反馈数据-通过id查询", notes="淘宝特价版客户反馈数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<BytedanceCustomerDdTbtjData> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BytedanceCustomerDdTbtjData> result = new Result<>();
		BytedanceCustomerDdTbtjData bytedanceCustomerDdTbtjData = bytedanceCustomerDdTbtjDataService.getById(id);
		if(bytedanceCustomerDdTbtjData==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(bytedanceCustomerDdTbtjData);
			result.setSuccess(true);
		}
		return result;
	}
  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @PostMapping(value = "/importExcel")
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
	  //1.读Excel
	  MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
	  Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
	  for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
		  FileInputStream fis = null;
		  try {
			  MultipartFile file = entity.getValue();
			  fis = (FileInputStream) file.getInputStream();
			  AnalysisEventListener listener = new BytedanceCustomTbtjDataExcelListener();
			  Boolean flag = ExcelUtils.readExcel(fis, BytedanceCustomTbtjDataEntity.class, listener);
			  System.out.println("导入是否成功：" + flag);
		  } catch (Exception e) {
			  e.printStackTrace();
		  } finally {
			  if (fis != null) {
				  try {
					  fis.close();
				  } catch (IOException e) {
					  e.printStackTrace();
				  }
			  }
		  }
	  }
	  return Result.ok("文件导入成功！");
  }

}
